# Project Overview

## Configuration
```json
{
  "directories": [
    "src"
  ],
  "output_file": "just-another-snapshot.md",
  "include_extensions": [
    ".py"
  ],
  "exclude_dirs": [
    "__pycache__",
    ".git",
    ".venv"
  ],
  "exclude_files": [
    "*.pyc"
  ]
}
```

## Directory Structure

```
/Users/ibrahimsaidi/Desktop/Builds/pypi_packages/llm-auto-context
├── src
│   └── llm_auto_context
│       ├── __init__.py
│       ├── __main__.py
│       ├── cli.py
│       ├── config.py
│       └── snapshot.py
└── tests
    ├── conftest.py
    ├── test_cli.py
    ├── test_config.py
    ├── test_data
    │   └── sample.py
    └── test_snapshot.py
```

# Source Code


## src/llm_auto_context/__init__.py

**Extension**: .py
**Size**: 116 bytes
**Last Modified**: 2025-02-10 08:27:41

```py
"""
LLM Auto Context - A CLI tool for generating code snapshots for LLM context windows.
"""

__version__ = "1.2.0" 
```

## src/llm_auto_context/__main__.py

**Extension**: .py
**Size**: 95 bytes
**Last Modified**: 2025-02-10 08:26:14

```py
"""Main entry point for the CLI."""
from .cli import app

if __name__ == "__main__":
    app() 
```

## src/llm_auto_context/cli.py

**Extension**: .py
**Size**: 3,358 bytes
**Last Modified**: 2025-02-04 08:15:51

```py
"""CLI interface for code snapshot generation."""

import json
import sys
from pathlib import Path
from typing import List, Optional

import typer
from typing_extensions import Annotated

from .config import SnapshotConfig
from .snapshot import create_code_snapshot

app = typer.Typer(help="Generate code snapshots with configurable settings")
DEFAULT_CONFIG = ".codesnapshot.json"

def load_config(config_path: Path) -> SnapshotConfig:
    """Load config from file or return defaults."""
    if not config_path.exists():
        raise FileNotFoundError(f"Config file not found: {config_path}")
    try:
        return SnapshotConfig.model_validate(json.loads(config_path.read_text()))
    except json.JSONDecodeError as e:
        raise ValueError(f"Invalid JSON in config file: {e}")
    except Exception as e:
        raise ValueError(f"Invalid config file: {e}")

@app.command()
def main(
    config: Annotated[
        Path,
        typer.Option(
            "--config",
            help="Path to config file",
            exists=False,
        )
    ] = Path(DEFAULT_CONFIG),
    directories: Annotated[
        Optional[List[str]],
        typer.Option(
            "--directory", "-d",
            help="Override directories to scan"
        )
    ] = None,
    output: Annotated[
        Optional[Path],
        typer.Option(
            "--output", "-o",
            help="Override output file path"
        )
    ] = None,
    include: Annotated[
        Optional[List[str]],
        typer.Option(
            "--include",
            help="Override file extensions to include"
        )
    ] = None,
    exclude_dir: Annotated[
        Optional[List[str]],
        typer.Option(
            "--exclude-dir",
            help="Additional directories to exclude"
        )
    ] = None,
    exclude_file: Annotated[
        Optional[List[str]],
        typer.Option(
            "--exclude-file",
            help="Additional files to exclude"
        )
    ] = None,
) -> None:
    """Generate code snapshots with configurable settings."""
    try:
        # Load config
        try:
            cfg = load_config(config)
        except FileNotFoundError:
            if not directories:
                typer.echo("Error: No config file found and no directories specified", err=True)
                raise typer.Exit(code=1)
            cfg = SnapshotConfig()
        
        # Override config with CLI options
        if directories:
            # Validate directories exist
            for directory in directories:
                if not Path(directory).exists():
                    typer.echo(f"Error: Directory not found: {directory}", err=True)
                    raise typer.Exit(code=1)
            cfg.directories = list(directories)
        if output:
            cfg.output_file = str(output)
        if include:
            cfg.include_extensions = list(include)
        if exclude_dir:
            cfg.exclude_dirs.extend(exclude_dir)
        if exclude_file:
            cfg.exclude_files.extend(exclude_file)

        # Create snapshot
        output_path = create_code_snapshot(cfg)
        typer.echo(f"Created snapshot at: {output_path}")

    except typer.Exit:
        raise
    except Exception as e:
        typer.echo(f"Error: {str(e)}", err=True)
        raise typer.Exit(code=1)

if __name__ == "__main__":
    app() 
```

## src/llm_auto_context/config.py

**Extension**: .py
**Size**: 2,043 bytes
**Last Modified**: 2025-02-04 08:15:51

```py
"""Configuration handling for code snapshot generation."""

from pathlib import Path
from typing import List, Optional
from pydantic import BaseModel, Field, field_validator

class SnapshotConfig(BaseModel):
    """Configuration model for code snapshot generation."""
    
    directories: List[str] = Field(
        default=["src"],
        description="List of directories to scan for code files"
    )
    output_file: str = Field(
        default="code_snapshot.txt",
        description="Path to output file"
    )
    include_extensions: List[str] = Field(
        default=[".swift", ".py", ".js"],
        description="File extensions to include"
    )
    exclude_dirs: List[str] = Field(
        default=["node_modules", ".git", "build"],
        description="Directories to exclude"
    )
    exclude_files: List[str] = Field(
        default=[],
        description="Specific files to exclude"
    )

    @field_validator("output_file")
    def validate_output_file(cls, v: str) -> str:
        """Validate output file path."""
        if not v:
            raise ValueError("Output file path cannot be empty")
        return v

    @field_validator("directories")
    def validate_directories(cls, v: List[str]) -> List[str]:
        """Validate directories list."""
        if not v:
            raise ValueError("At least one directory must be specified")
        return v

    def get_output_path(self, base_dir: Optional[Path] = None) -> Path:
        """Get absolute output path, optionally relative to base_dir."""
        path = Path(self.output_file)
        if base_dir and not path.is_absolute():
            path = base_dir / path
        return path.resolve()

    class Config:
        json_schema_extra = {
            "example": {
                "directories": ["src", "lib"],
                "output_file": "snapshot.md",
                "include_extensions": [".py", ".js", ".ts"],
                "exclude_dirs": ["node_modules", ".git"],
                "exclude_files": ["secrets.env"]
            }
        } 
```

## src/llm_auto_context/snapshot.py

**Extension**: .py
**Size**: 6,233 bytes
**Last Modified**: 2025-02-10 08:50:54

```py
"""Core functionality for generating code snapshots."""

import os
import subprocess
from pathlib import Path
from typing import Set, Dict
from datetime import datetime

from .config import SnapshotConfig

def should_include_file(
    file_path: Path,
    config: SnapshotConfig,
    base_dir: Path
) -> bool:
    """Check if a file should be included in the snapshot."""
    # Check if file is in exclude list
    rel_path = str(file_path.relative_to(base_dir))
    if any(exclude in rel_path for exclude in config.exclude_files):
        return False

    # Check if file is in excluded directory
    for exclude_dir in config.exclude_dirs:
        if exclude_dir in rel_path.split(os.sep):
            return False

    # Check file extension
    return file_path.suffix.lower() in config.include_extensions

def get_project_tree(base_dir: Path, config: SnapshotConfig) -> str:
    """Get project tree structure using tree command."""
    try:
        # Common development files/dirs to exclude from tree
        common_excludes = [
            ".git", ".pytest_cache", "__pycache__", ".coverage",
            ".env", ".venv", "*.pyc", "*.pyo", "*.pyd", "*.so",
            "build", "dist", "*.egg-info"
        ]
        
        # Build exclude patterns for tree command
        exclude_patterns = []
        
        # Add common excludes
        for exclude in common_excludes:
            pattern = exclude.replace("*", "")
            if pattern:
                exclude_patterns.extend(["-I", pattern])
        
        # Add user-configured excludes
        for exclude in config.exclude_files:
            pattern = exclude.replace("*", "")
            if pattern:
                exclude_patterns.extend(["-I", pattern])
        
        for exclude_dir in config.exclude_dirs:
            exclude_patterns.extend(["-I", exclude_dir])
            
        # Only show files with included extensions
        include_patterns = []
        for ext in config.include_extensions:
            include_patterns.append(f"*{ext}")
        pattern_str = "|".join(include_patterns)
        
        # Run tree command with proper flags
        cmd = ["tree", str(base_dir), "-a", "--noreport", "--prune", "-P", pattern_str] + exclude_patterns
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return result.stdout.strip()
    except subprocess.CalledProcessError as e:
        print(f"Warning: Failed to generate tree structure: {e}")
        return "Failed to generate tree structure"
    except FileNotFoundError:
        print("Warning: 'tree' command not found. Please install it to see project structure.")
        return "Tree command not available"

def create_code_snapshot(config: SnapshotConfig) -> Path:
    """Create a code snapshot based on configuration."""
    base_dir = Path.cwd()
    output_path = config.get_output_path(base_dir)
    output_path.parent.mkdir(parents=True, exist_ok=True)

    processed_files: Set[Path] = set()
    file_index: Dict[str, str] = {}

    with open(output_path, "w", encoding="utf-8") as out_f:
        # Project Overview
        out_f.write("# Project Overview\n\n")
        
        # Configuration
        out_f.write("## Configuration\n```json\n")
        out_f.write(config.model_dump_json(indent=2))
        out_f.write("\n```\n\n")
        
        # Project Structure
        out_f.write("## Directory Structure\n\n```\n")
        tree_output = get_project_tree(base_dir, config)
        out_f.write(tree_output)
        out_f.write("\n```\n\n")
        
        # Source Code Section
        out_f.write("# Source Code\n\n")
        
        for directory in config.directories:
            dir_path = Path(directory).resolve()
            if not dir_path.exists():
                print(f"Warning: Directory {directory} does not exist, skipping.")
                continue

            # Walk through directory structure
            for root, _, files in os.walk(dir_path):
                root_path = Path(root)
                
                # Sort files for consistency
                for file_name in sorted(files):
                    file_path = root_path / file_name
                    
                    # Skip if already processed or shouldn't be included
                    if (file_path in processed_files or
                        file_path.name.startswith('.') or
                        not should_include_file(file_path, config, base_dir)):
                        continue

                    # Get file metadata
                    rel_path = file_path.relative_to(base_dir)
                    file_stats = file_path.stat()
                    last_modified = datetime.fromtimestamp(file_stats.st_mtime)
                    
                    # Add to file index
                    file_index[str(rel_path)] = f"[{file_path.suffix.lstrip('.')}] {file_stats.st_size:,} bytes - {last_modified.strftime('%Y-%m-%d %H:%M:%S')}"

                    # Write file section with metadata
                    out_f.write(f"\n## {rel_path}\n\n")
                    out_f.write(f"**Extension**: {file_path.suffix}\n")
                    out_f.write(f"**Size**: {file_stats.st_size:,} bytes\n")
                    out_f.write(f"**Last Modified**: {last_modified.strftime('%Y-%m-%d %H:%M:%S')}\n\n")
                    
                    # Write file content with language-specific syntax highlighting
                    lang = file_path.suffix.lstrip('.')
                    out_f.write(f"```{lang}\n")
                    
                    try:
                        content = file_path.read_text(encoding="utf-8", errors="ignore")
                        out_f.write(content)
                        out_f.write("\n```\n")
                        processed_files.add(file_path)
                    except Exception as e:
                        print(f"Error reading {file_path}: {e}")
                        out_f.write(f"Error reading file: {e}\n```\n")

        # Write File Index after processing all files
        out_f.write("\n## File Index\n\n")
        for file_path, metadata in sorted(file_index.items()):
            out_f.write(f"- `{file_path}` - {metadata}\n")

    return output_path 
```

## File Index

- `src/llm_auto_context/__init__.py` - [py] 116 bytes - 2025-02-10 08:27:41
- `src/llm_auto_context/__main__.py` - [py] 95 bytes - 2025-02-10 08:26:14
- `src/llm_auto_context/cli.py` - [py] 3,358 bytes - 2025-02-04 08:15:51
- `src/llm_auto_context/config.py` - [py] 2,043 bytes - 2025-02-04 08:15:51
- `src/llm_auto_context/snapshot.py` - [py] 6,233 bytes - 2025-02-10 08:50:54
