"""Core functionality for generating code snapshots."""

import os
import subprocess
from pathlib import Path
from typing import Set, Dict
from datetime import datetime

from .config import SnapshotConfig

def should_include_file(
    file_path: Path,
    config: SnapshotConfig,
    base_dir: Path
) -> bool:
    """Check if a file should be included in the snapshot."""
    # Check if file is in exclude list
    rel_path = str(file_path.relative_to(base_dir))
    if any(exclude in rel_path for exclude in config.exclude_files):
        return False

    # Check if file is in excluded directory
    for exclude_dir in config.exclude_dirs:
        if exclude_dir in rel_path.split(os.sep):
            return False

    # Check file extension
    return file_path.suffix.lower() in config.include_extensions

def get_project_tree(base_dir: Path, config: SnapshotConfig) -> str:
    """Get project tree structure using tree command."""
    try:
        # Common development files/dirs to exclude from tree
        common_excludes = [
            ".git", ".pytest_cache", "__pycache__", ".coverage",
            ".env", ".venv", "*.pyc", "*.pyo", "*.pyd", "*.so",
            "build", "dist", "*.egg-info"
        ]
        
        # Build exclude patterns for tree command
        exclude_patterns = []
        
        # Add common excludes
        for exclude in common_excludes:
            pattern = exclude.replace("*", "")
            if pattern:
                exclude_patterns.extend(["-I", pattern])
        
        # Add user-configured excludes
        for exclude in config.exclude_files:
            pattern = exclude.replace("*", "")
            if pattern:
                exclude_patterns.extend(["-I", pattern])
        
        for exclude_dir in config.exclude_dirs:
            exclude_patterns.extend(["-I", exclude_dir])
            
        # Only show files with included extensions
        include_patterns = []
        for ext in config.include_extensions:
            include_patterns.append(f"*{ext}")
        pattern_str = "|".join(include_patterns)
        
        # Run tree command with proper flags
        cmd = ["tree", str(base_dir), "-a", "--noreport", "--prune", "-P", pattern_str] + exclude_patterns
        result = subprocess.run(cmd, capture_output=True, text=True, check=True)
        return result.stdout.strip()
    except subprocess.CalledProcessError as e:
        print(f"Warning: Failed to generate tree structure: {e}")
        return "Failed to generate tree structure"
    except FileNotFoundError:
        print("Warning: 'tree' command not found. Please install it to see project structure.")
        return "Tree command not available"

def create_code_snapshot(config: SnapshotConfig) -> Path:
    """Create a code snapshot based on configuration."""
    base_dir = Path.cwd()
    output_path = config.get_output_path(base_dir)
    output_path.parent.mkdir(parents=True, exist_ok=True)

    processed_files: Set[Path] = set()
    file_index: Dict[str, str] = {}

    with open(output_path, "w", encoding="utf-8") as out_f:
        # Project Overview
        out_f.write("# Project Overview\n\n")
        
        # Configuration
        out_f.write("## Configuration\n```json\n")
        out_f.write(config.model_dump_json(indent=2))
        out_f.write("\n```\n\n")
        
        # Project Structure
        out_f.write("## Directory Structure\n\n```\n")
        tree_output = get_project_tree(base_dir, config)
        out_f.write(tree_output)
        out_f.write("\n```\n\n")
        
        # Source Code Section
        out_f.write("# Source Code\n\n")
        
        for directory in config.directories:
            dir_path = Path(directory).resolve()
            if not dir_path.exists():
                print(f"Warning: Directory {directory} does not exist, skipping.")
                continue

            # Walk through directory structure
            for root, _, files in os.walk(dir_path):
                root_path = Path(root)
                
                # Sort files for consistency
                for file_name in sorted(files):
                    file_path = root_path / file_name
                    
                    # Skip if already processed or shouldn't be included
                    if (file_path in processed_files or
                        file_path.name.startswith('.') or
                        not should_include_file(file_path, config, base_dir)):
                        continue

                    # Get file metadata
                    rel_path = file_path.relative_to(base_dir)
                    file_stats = file_path.stat()
                    last_modified = datetime.fromtimestamp(file_stats.st_mtime)
                    
                    # Add to file index
                    file_index[str(rel_path)] = f"[{file_path.suffix.lstrip('.')}] {file_stats.st_size:,} bytes - {last_modified.strftime('%Y-%m-%d %H:%M:%S')}"

                    # Write file section with metadata
                    out_f.write(f"\n## {rel_path}\n\n")
                    out_f.write(f"**Extension**: {file_path.suffix}\n")
                    out_f.write(f"**Size**: {file_stats.st_size:,} bytes\n")
                    out_f.write(f"**Last Modified**: {last_modified.strftime('%Y-%m-%d %H:%M:%S')}\n\n")
                    
                    # Write file content with language-specific syntax highlighting
                    lang = file_path.suffix.lstrip('.')
                    out_f.write(f"```{lang}\n")
                    
                    try:
                        content = file_path.read_text(encoding="utf-8", errors="ignore")
                        out_f.write(content)
                        out_f.write("\n```\n")
                        processed_files.add(file_path)
                    except Exception as e:
                        print(f"Error reading {file_path}: {e}")
                        out_f.write(f"Error reading file: {e}\n```\n")

        # Write File Index after processing all files
        out_f.write("\n## File Index\n\n")
        for file_path, metadata in sorted(file_index.items()):
            out_f.write(f"- `{file_path}` - {metadata}\n")

    return output_path 